CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
125 lines
3.8 KiB
Plaintext
125 lines
3.8 KiB
Plaintext
// Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
//
|
|
// SPDX-License-Identifier: MIT
|
|
|
|
#include <hip/hip_runtime.h>
|
|
#include <hip/hip_complex.h>
|
|
|
|
#include <cmath>
|
|
#include <cstdlib>
|
|
#include <iostream>
|
|
#include <vector>
|
|
|
|
#define HIP_CHECK(expression) \
|
|
{ \
|
|
const hipError_t err = expression; \
|
|
if (err != hipSuccess) { \
|
|
std::cerr << "HIP error: " \
|
|
<< hipGetErrorString(err) \
|
|
<< " at " << __LINE__ << "\n"; \
|
|
exit(EXIT_FAILURE); \
|
|
} \
|
|
}
|
|
|
|
// Kernel to compute DFT
|
|
__global__ void computeDFT(const float* input, hipFloatComplex* output, const int N)
|
|
{
|
|
int k = blockIdx.x * blockDim.x + threadIdx.x;
|
|
if (k >= N) return;
|
|
|
|
hipFloatComplex sum = make_hipFloatComplex(0.0f, 0.0f);
|
|
|
|
for (int n = 0; n < N; n++)
|
|
{
|
|
float angle = -2.0f * M_PI * k * n / N;
|
|
hipFloatComplex w = make_hipFloatComplex(cosf(angle), sinf(angle));
|
|
hipFloatComplex x = make_hipFloatComplex(input[n], 0.0f);
|
|
sum = hipCaddf(sum, hipCmulf(x, w));
|
|
}
|
|
|
|
output[k] = sum;
|
|
}
|
|
|
|
// CPU implementation of DFT for verification
|
|
std::vector<hipFloatComplex> cpuDFT(const std::vector<float>& input)
|
|
{
|
|
const int N = input.size();
|
|
std::vector<hipFloatComplex> result(N);
|
|
|
|
for (int k = 0; k < N; k++)
|
|
{
|
|
hipFloatComplex sum = make_hipFloatComplex(0.0f, 0.0f);
|
|
for (int n = 0; n < N; n++)
|
|
{
|
|
float angle = -2.0f * M_PI * k * n / N;
|
|
hipFloatComplex w = make_hipFloatComplex(cosf(angle), sinf(angle));
|
|
hipFloatComplex x = make_hipFloatComplex(input[n], 0.0f);
|
|
sum = hipCaddf(sum, hipCmulf(x, w));
|
|
}
|
|
result[k] = sum;
|
|
}
|
|
return result;
|
|
}
|
|
|
|
int main()
|
|
{
|
|
const int N = 256; // Signal length
|
|
const int blockSize = 256;
|
|
|
|
// Generate input signal: sum of two sine waves
|
|
std::vector<float> signal(N);
|
|
for (int i = 0; i < N; i++)
|
|
{
|
|
float t = static_cast<float>(i) / N;
|
|
signal[i] = sinf(2.0f * M_PI * 10.0f * t) + // 10 Hz component
|
|
0.5f * sinf(2.0f * M_PI * 20.0f * t); // 20 Hz component
|
|
}
|
|
|
|
// Compute reference solution on CPU
|
|
std::vector<hipFloatComplex> cpu_output = cpuDFT(signal);
|
|
|
|
// Allocate device memory
|
|
float* d_signal;
|
|
hipFloatComplex* d_output;
|
|
HIP_CHECK(hipMalloc(&d_signal, N * sizeof(float)));
|
|
HIP_CHECK(hipMalloc(&d_output, N * sizeof(hipFloatComplex)));
|
|
|
|
// Copy input to device
|
|
HIP_CHECK(hipMemcpy(d_signal, signal.data(), N * sizeof(float), hipMemcpyHostToDevice));
|
|
|
|
// Launch kernel
|
|
dim3 grid((N + blockSize - 1) / blockSize);
|
|
dim3 block(blockSize);
|
|
computeDFT<<<grid, block>>>(d_signal, d_output, N);
|
|
HIP_CHECK(hipGetLastError());
|
|
|
|
// Get GPU results
|
|
std::vector<hipFloatComplex> gpu_output(N);
|
|
HIP_CHECK(hipMemcpy(gpu_output.data(), d_output, N * sizeof(hipFloatComplex), hipMemcpyDeviceToHost));
|
|
|
|
// Verify results
|
|
bool passed = true;
|
|
const float tolerance = 1e-5f; // Adjust based on precision requirements
|
|
|
|
for (int i = 0; i < N; i++)
|
|
{
|
|
float diff_real = std::abs(hipCrealf(gpu_output[i]) - hipCrealf(cpu_output[i]));
|
|
float diff_imag = std::abs(hipCimagf(gpu_output[i]) - hipCimagf(cpu_output[i]));
|
|
|
|
if (diff_real > tolerance || diff_imag > tolerance)
|
|
{
|
|
passed = false;
|
|
break;
|
|
}
|
|
}
|
|
|
|
std::cout << "DFT Verification: " << (passed ? "PASSED" : "FAILED") << "\n";
|
|
|
|
// Cleanup
|
|
HIP_CHECK(hipFree(d_signal));
|
|
HIP_CHECK(hipFree(d_output));
|
|
|
|
return passed ? EXIT_SUCCESS : EXIT_FAILURE;
|
|
}
|
|
// [sphinx-end]
|